@@ -1,4 +1,4 @@
-#! /usr/bin/env python3
+#!/usr/pkg/bin/python2.7
 
 """Token constants (from "token.h")."""
 
@@ -68,7 +68,7 @@ NT_OFFSET = 256
 #--end constants--
 
 tok_name = {}
-for _name, _value in list(globals().items()):
+for _name, _value in globals().items():
     if type(_value) is type(0):
         tok_name[_value] = _name
 
